// Log.cpp : implementation file
//

#include "stdafx.h"
#include "MDITest.h"
#include "printer.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

/////////////////////////////////////////////////////////////////////////////
// CLog dialog


CLog::CLog(CWnd* pParent /*=NULL*/)
	: CDialog(CLog::IDD, pParent)
{
	//{{AFX_DATA_INIT(CLog)
		// NOTE: the ClassWizard will add member initialization here
	//}}AFX_DATA_INIT
}


void CLog::DoDataExchange(CDataExchange* pDX)
{
	CDialog::DoDataExchange(pDX);
	//{{AFX_DATA_MAP(CLog)
	DDX_Control(pDX, IDC_LOG, c_Log);
	//}}AFX_DATA_MAP
}


BEGIN_MESSAGE_MAP(CLog, CDialog)
        ON_MESSAGE(UWM_LOG, OnLog)
	ON_MESSAGE(UWM_CLEAR, OnClear)
	//{{AFX_MSG_MAP(CLog)
	ON_WM_CLOSE()
	ON_WM_SIZE()
	ON_COMMAND(IDM_CLEAR, OnClear)
	ON_COMMAND(IDM_PRINT, OnPrint)
	ON_COMMAND(IDM_SAVE, OnSave)
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// CLog message handlers

/****************************************************************************
*                            CLog::updateControls
* Result: void
*       
* Effect: 
*       Updates the controls
****************************************************************************/

void CLog::updateControls()
    {
     BOOL enable = c_Log.GetCount() > 0;
     GetMenu()->EnableMenuItem(IDM_PRINT, enable ? MF_ENABLED : MF_GRAYED);
     GetMenu()->EnableMenuItem(IDM_CLEAR, enable ? MF_ENABLED : MF_GRAYED);
    } // CLog::updateControls

void CLog::OnOK() 
{
}

void CLog::OnCancel() 
{
}

void CLog::OnClose() 
   {
   }

BOOL CLog::OnInitDialog() 
{
	CDialog::OnInitDialog();
	
	CFont f;
	f.CreateFont(-11,             // height
		     0,               // width
		     0,               // escapement
		     0,               // orientation
		     FW_NORMAL,       // weight
		     FALSE,           // italic
		     FALSE,	      // underline
		     FALSE,           // strikeout
		     ANSI_CHARSET,    // character set
		     OUT_TT_PRECIS,   // output precision
		     CLIP_TT_ALWAYS,  // clip precision
		     DEFAULT_QUALITY, // quality
		     FIXED_PITCH | FF_DONTCARE, // pitch & family
		     _T("Courier New")); // font
	c_Log.SetFont(&f);
	f.Detach();
	updateControls();
	
	return TRUE;  // return TRUE unless you set the focus to a control
	              // EXCEPTION: OCX Property Pages should return FALSE
}

/****************************************************************************
*                                 CLog::OnLog
* Inputs:
*       WPARAM: (WPARAM)(BOOL) TRUE if terminal, FALSE if not
*	LPARAM: (LPARAM)(LPCTSTR) pointer to string
* Result: LRESULT
*       Logically void, 0, always
* Effect: 
*       Logs the item
****************************************************************************/

LRESULT CLog::OnLog(WPARAM wParam, LPARAM lParam)
    {
     CString * s = (CString *)lParam;
     c_Log.AddString(*s);
     if(!(BOOL)wParam)
	c_Log.AddString(CString(_T('-'), 80));
     delete s;
     updateControls();
     return 0;
    } // CLog::OnLog

/****************************************************************************
*                                CLog::OnClear
* Inputs:
*       WPARAM: ignored
*	LPARAM: ignored
* Result: LRESULT
*       0
* Effect: 
*       Clears the control
****************************************************************************/

LRESULT CLog::OnClear(WPARAM, LPARAM)
    {
     c_Log.ResetContent();
     updateControls();
     return 0;
    } // CLog::OnClear

/****************************************************************************
*                                CLog::OnSize
* Inputs:
*       UINT nType: Size type
*	int cx: New client area x
*	int cy: New client area y
* Result: void
*       
* Effect: 
*       Resizes the contained control
****************************************************************************/

void CLog::OnSize(UINT nType, int cx, int cy) 
   {
    CDialog::OnSize(nType, cx, cy);
	
    if(c_Log.GetSafeHwnd() != NULL)
       { /* resize */
	c_Log.SetWindowPos(NULL, 0, 0, cx, cy,
			   SWP_NOMOVE | SWP_NOZORDER);
       } /* resize */
	
}

void CLog::OnClear() 
   {
    c_Log.ResetContent();
    updateControls();
   }

void CLog::OnPrint() 
   {
    CPrintDialog printdlg(FALSE, PD_RETURNDC, this);
    if(printdlg.DoModal() != IDOK)
       return;
    CPrinter printer(CDC::FromHandle(printdlg.m_pd.hDC));
    printer.StartPrinting();
    for(int i = 0; i < c_Log.GetCount(); i++)
       { /* print each line */
	CString s;
	c_Log.GetText(i, s);
	printer.PrintLine(s);
       } /* print each line */
    printer.EndPrinting();
   }

/****************************************************************************
*                                CLog::OnSave
* Result: void
*       
* Effect: 
*       Saves the file
****************************************************************************/

void CLog::OnSave() 
   {
    CString filter;
    filter.LoadString(IDS_TRACELOG_FILTER);
    CFileDialog dlg(FALSE, _T("log"), NULL,
		    OFN_HIDEREADONLY | OFN_OVERWRITEPROMPT,
		    filter);
	
    switch(dlg.DoModal())
       { /* domodal */
	case 0: // should never happen
	case IDCANCEL:
	   return;
       } /* domodal */
    
    CStdioFile f;
    if(f.Open(dlg.GetPathName(), CFile::modeCreate | CFile::modeWrite))
       { /* save */
	for(int i = 0; i < c_Log.GetCount(); i++)
	   { /* save each */
	    CString s;
	    c_Log.GetText(i, s);
	    f.WriteString(s);
	    f.WriteString(_T("\r\n"));
	   } /* save each */
	f.Close();
       } /* save */
    else
       { /* save failed */
        DWORD err = ::GetLastError();
	AfxMessageBox(IDS_OPEN_FAILED, MB_ICONERROR | MB_OK);
       } /* save failed */
   }
